Permit

Struct Permit 

pub struct Permit {
    bucket: Arc<AtomicTokenBucket>,
    granted: u64,
}
Proof that the requested number of tokens were granted.

If dropped, any remaining tokens will be refunded hence why there is a reference to the shared bucket.

Fields

bucket: Arc<AtomicTokenBucket>

The bucket the unclaimed tokens are refunded to when this permit is dropped.

granted: u64

How many granted tokens this permit still has.

Lowered by the claim methods. What is left is refunded on drop.

Implementations

Source

impl Permit

fn new(bucket: Arc<AtomicTokenBucket>, granted: u64) -> Self

Construct a permit granting granted tokens.

We keep a reference to the bucket for a refund if any.

pub fn claim(&mut self, tokens: u64) -> Result<(), BwPoolError>

Claim tokens from this permit.

Returns true if the permit had at least tokens that are now claimed. Else, return false and nothing is claimed as in the caller is trying to use more than it was granted.

pub fn claim_all(&mut self)

Claim everything this permit holds.

Used in the for a Sink implementation that gets its permit in the poll_ready() which is before the start_send().

pub fn granted(&self) -> u64

How many tokens this permit still has granted a.k.a available.