poll_read_limited

Function poll_read_limited 

pub(super) fn poll_read_limited<R: AsyncRead>(
    inner: Pin<&mut R>,
    state: &mut DirectionState,
    cx: &mut Context<'_>,
    buf: &mut [u8],
) -> Poll<Result<usize, Error>>
Helper: Rate-limited AsyncRead::poll_read of inner using the given read direction state. This is used by multiple object hence why in a read helper.