poll_write_limited

Function poll_write_limited 

pub(super) fn poll_write_limited<W: AsyncWrite>(
    inner: Pin<&mut W>,
    state: &mut DirectionState,
    cx: &mut Context<'_>,
    buf: &[u8],
) -> Poll<Result<usize, Error>>
Helper: Rate-limited AsyncWrite::poll_write of inner using the given read direction state. This is used by multiple object hence why in a write helper.