Sender

Struct Sender 

pub struct Sender<T> {
    shared: Weak<Shared<T>>,
}
Fields

shared: Weak<Shared<T>>

State shared with all Receivers.

Implementations

Source

impl<T> Sender<T>

pub fn send(self, msg: T)

Send the message to the Receivers.

The message may be lost if all receivers have been dropped.

fn send_and_wake( shared: &Weak<Shared<T>>, msg: Result<T, SenderDropped>, ) -> Result<(), MessageAlreadySet>

Send the message, and wake and clear all wakers.

If all receivers have been dropped, then always returns Ok.

If the message was unable to be set, returns Err(MessageAlreadySet).

pub fn is_cancelled(&self) -> bool

Returns true if all Receivers (and all futures created from the receivers) have been dropped.

This can be useful to skip doing extra work to generate the message if the message will be discarded anyways.

pub fn subscribe(&self) -> Result<Receiver<T>, AllReceiversDropped>

Subscribe to this channel, creating a new receiver.

Returns an error if all the existing Receivers (and all futures created from the receivers) have already been dropped.