#[repr(transparent)]struct WakerKey(KeyData);Tuple Fields
0: KeyDataTrait Implementations
impl Copy for WakerKey
Sourceimpl<'de> Deserialize<'de> for WakerKey
impl<'de> Deserialize<'de> for WakerKey
Sourcefn deserialize<D>(deserializer: D) -> Result<Self, D::Error>where
D: Deserializer<'de>,
fn deserialize<D>(deserializer: D) -> Result<Self, D::Error>where
D: Deserializer<'de>,
Deserialize this value from the given Serde deserializer. Read more
impl Eq for WakerKey
Sourceimpl Key for WakerKey
impl Key for WakerKey
Sourcefn null() -> Self
fn null() -> Self
Creates a new key that is always invalid and distinct from any non-null
key. A null key can only be created through this method (or default
initialization of keys made with
new_key_type!, which calls this
method). Read moreSourceimpl Ord for WakerKey
impl Ord for WakerKey
1.21.0 (const: unstable) · Sourcefn max(self, other: Self) -> Selfwhere
Self: Sized,
fn max(self, other: Self) -> Selfwhere
Self: Sized,
Compares and returns the maximum of two values. Read more
1.21.0 (const: unstable) · Sourcefn min(self, other: Self) -> Selfwhere
Self: Sized,
fn min(self, other: Self) -> Selfwhere
Self: Sized,
Compares and returns the minimum of two values. Read more
Sourceimpl PartialOrd for WakerKey
impl PartialOrd for WakerKey
impl StructuralPartialEq for WakerKey