WakersAlreadyWoken

Struct WakersAlreadyWoken 

struct WakersAlreadyWoken;
The wakers have already been woken.

This is used to help detect if we’re trying to access the wakers after they’ve already been woken, which likely indicates a bug. For example, it is a bug if a receiver attempts to add a waker after the sender has already sent its message and woken the wakers, since the new waker would never be woken.