macro_rules! dprintln {
{ $f:literal $($a:tt)* } => { ... };
}Expand description
Switch to the nontrivial version of this, to get debugging output on stderr
macro_rules! dprintln {
{ $f:literal $($a:tt)* } => { ... };
}Switch to the nontrivial version of this, to get debugging output on stderr