pub struct Permit {
bucket: Arc<AtomicTokenBucket>,
granted: u64,
}Expand description
Proof that the requested number of tokens were granted.
If dropped, any remaining tokens will be refunded hence why there is a reference to the shared bucket.
Fields
bucket: Arc<AtomicTokenBucket>The bucket the unclaimed tokens are refunded to when this permit is dropped.
granted: u64How many granted tokens this permit still has.
Lowered by the claim methods. What is left is refunded on drop.
Implementations
Sourceimpl Permit
impl Permit
Sourcefn new(bucket: Arc<AtomicTokenBucket>, granted: u64) -> Self
fn new(bucket: Arc<AtomicTokenBucket>, granted: u64) -> Self
Construct a permit granting granted tokens.
We keep a reference to the bucket for a refund if any.
Sourcepub fn claim(&mut self, tokens: u64) -> Result<(), BwPoolError>
pub fn claim(&mut self, tokens: u64) -> Result<(), BwPoolError>
Claim tokens from this permit.
Returns true if the permit had at least tokens that are now claimed. Else,
return false and nothing is claimed as in the caller is trying to use more
than it was granted.