pub enum ErasedSinkTrySendError {
Full,
Disconnected,
Other(Arc<dyn Error + Send + Sync + 'static>),
}Expand description
Type-erased error for SinkTrySend::try_send
Provided for situations where providing a concrete error type is awkward.
futures::channel::mpsc::Sender wants this because when its try_send method fails,
it is not possible to extract both the sent item, and the error!
tor_memquota::mq_queue::Sender wants this because the types of the error return
from its try_sendwould otherwise be tainted by complex generics, including its privateEntry` type.
Variants
Full
The stream was full.
No arrangements will have been made for a wakeup when space becomes available.
Corresponds to SinkTrySendError::is_full
Disconnected
The stream has disconnected
Corresponds to SinkTrySendError::is_disconnected
Other(Arc<dyn Error + Send + Sync + 'static>)
Something else went wrong
Implementations
Sourceimpl ErasedSinkTrySendError
impl ErasedSinkTrySendError
Sourcepub fn from<E>(e: E) -> ErasedSinkTrySendError
pub fn from<E>(e: E) -> ErasedSinkTrySendError
Obtain an ErasedSinkTrySendError from a concrete SinkTrySendError