pub struct Sender<T> {
shared: Weak<Shared<T>>,
}Expand description
A oneshot broadcast sender.
Fields
State shared with all Receivers.
Implementations
Sourceimpl<T> Sender<T>
impl<T> Sender<T>
Sourcepub fn send(self, msg: T)
pub fn send(self, msg: T)
Send the message to the Receivers.
The message may be lost if all receivers have been dropped.
Sourcefn send_and_wake(
shared: &Weak<Shared<T>>,
msg: Result<T, SenderDropped>,
) -> Result<(), MessageAlreadySet>
fn send_and_wake( shared: &Weak<Shared<T>>, msg: Result<T, SenderDropped>, ) -> Result<(), MessageAlreadySet>
Send the message, and wake and clear all wakers.
If all receivers have been dropped, then always returns Ok.
If the message was unable to be set, returns Err(MessageAlreadySet).
Sourcepub fn is_cancelled(&self) -> bool
pub fn is_cancelled(&self) -> bool
Returns true if all Receivers (and all futures created from the receivers) have been
dropped.
This can be useful to skip doing extra work to generate the message if the message will be discarded anyways.