Sender

Struct Sender 

pub struct Sender<T> { /* private fields */ }
Implementations

Source

impl<T> Sender<T>

pub fn send(self, msg: T)

Send the message to the Receivers.

The message may be lost if all receivers have been dropped.

pub fn is_cancelled(&self) -> bool

Returns true if all Receivers (and all futures created from the receivers) have been dropped.

This can be useful to skip doing extra work to generate the message if the message will be discarded anyways.