pub struct Sender<T> { /* private fields */ }Expand description
A oneshot broadcast sender.
Implementations
Sourceimpl<T> Sender<T>
impl<T> Sender<T>
Sourcepub fn send(self, msg: T)
pub fn send(self, msg: T)
Send the message to the Receivers.
The message may be lost if all receivers have been dropped.
Sourcepub fn is_cancelled(&self) -> bool
pub fn is_cancelled(&self) -> bool
Returns true if all Receivers (and all futures created from the receivers) have been
dropped.
This can be useful to skip doing extra work to generate the message if the message will be discarded anyways.