pub struct SinkPrepareSendFuture<'w, IF, OS, OM> {
generator: IF,
output: Option<Pin<&'w mut OS>>,
tw: PhantomData<fn(OM)>,
}Expand description
Future for SinkPrepareExt::prepare_send_from
Fields
generator: IFUnderlying future that will yield a message.
output: Option<Pin<&'w mut OS>>This Option exists because otherwise SinkPrepareSendFuture::poll()
can’t move output out of this struct to put it into the SinkSendable.
(The poll() impl cannot borrow from SinkPrepareSendFuture.)
tw: PhantomData<fn(OM)>fn(OM) gives contravariance in OM.
Variance is confusing. Loosely, a SinkPrepareSendFuture<..OM> consumes an OM. Actually, we don’t really need to add any variance restricions wrt OM, because the &mut OS already implies the correct variance, so we could have used the PhantomData<fn(*const OM)> trick. Happily there is no unsafe anywhere nearby, so it is not possible for us to write a bug due to getting the variance wrong - only to erroneously prevent some use case.
Implementations
Sourceimpl<'w, IF, OS, OM> SinkPrepareSendFuture<'w, IF, OS, OM>
impl<'w, IF, OS, OM> SinkPrepareSendFuture<'w, IF, OS, OM>